[{"locale":"es"},{"key":"03B1","mappings":{"default":{"default":"alfa"}},"category":"Ll"},{"key":"03B2","mappings":{"default":{"default":"beta"}},"category":"Ll"},{"key":"03B3","mappings":{"default":{"default":"gamma"}},"category":"Ll"},{"key":"03B4","mappings":{"default":{"default":"delta"}},"category":"Ll"},{"key":"03B5","mappings":{"default":{"default":"épsilon"}},"category":"Ll"},{"key":"03B6","mappings":{"default":{"default":"zeta"}},"category":"Ll"},{"key":"03B7","mappings":{"default":{"default":"eta"}},"category":"Ll"},{"key":"03B8","mappings":{"default":{"default":"theta"}},"category":"Ll"},{"key":"03B9","mappings":{"default":{"default":"iota"}},"category":"Ll"},{"key":"03BA","mappings":{"default":{"default":"kappa"}},"category":"Ll"},{"key":"03BB","mappings":{"default":{"default":"lambda"}},"category":"Ll"},{"key":"03BC","mappings":{"default":{"default":"mi"}},"category":"Ll"},{"key":"03BD","mappings":{"default":{"default":"ni"}},"category":"Ll"},{"key":"03BE","mappings":{"default":{"default":"xi"}},"category":"Ll"},{"key":"03BF","mappings":{"default":{"default":"ómicron"}},"category":"Ll"},{"key":"03C0","mappings":{"default":{"default":"pi"}},"category":"Ll"},{"key":"03C1","mappings":{"default":{"default":"rho"}},"category":"Ll"},{"key":"03C2","mappings":{"default":{"default":"sigma final"}},"category":"Ll"},{"key":"03C3","mappings":{"default":{"default":"sigma"}},"category":"Ll"},{"key":"03C4","mappings":{"default":{"default":"tau"}},"category":"Ll"},{"key":"03C5","mappings":{"default":{"default":"ípsilon"}},"category":"Ll"},{"key":"03C6","mappings":{"default":{"default":"phi"}},"category":"Ll"},{"key":"03C7","mappings":{"default":{"default":"ji"}},"category":"Ll"},{"key":"03C8","mappings":{"default":{"default":"psi"}},"category":"Ll"},{"key":"03C9","mappings":{"default":{"default":"omega"}},"category":"Ll"}]
