# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""Spyder API Version."""

VERSION_INFO = (0, 1, 0)
__version__ = '.'.join(map(str, VERSION_INFO))
